#include <windows.h>
#include "../IHexControlPlugin.h"
#include "resource.h"

BOOL DllInitialize();
BOOL DllUninitialize();

HINSTANCE DllhInstance;
BOOL WINAPI DllMain(HINSTANCE hInstDLL, DWORD dwReason, PVOID pvReserved)
{
    UNREFERENCED_PARAMETER(pvReserved);
    switch (dwReason)
    {
    case DLL_PROCESS_ATTACH:
        DllhInstance=hInstDLL;
        return DllInitialize();
    case DLL_PROCESS_DETACH:
        return DllUninitialize();
    }
    return TRUE;
}

BOOL DllInitialize()
{
    // Do any global var initialization here
    return TRUE;
}
BOOL DllUninitialize()
{
    // Do any global var uninitialization here. 
    return TRUE;
}

// function helper to load plugin menu icon
HICON LoadIcon(UINT IdIcon)
{
    return (HICON)::LoadImage(DllhInstance, MAKEINTRESOURCE(IdIcon),IMAGE_ICON,0,0,LR_SHARED);
}

// The Plugin object to implement
class CPlugin:public CHeliumHexControlEventsBase
{
public:
    CPlugin();
    virtual ~CPlugin();

    ////////////////////////////////////////////////
    // Plugin events
    ////////////////////////////////////////////////
    virtual BOOL STDMETHODCALLTYPE OnLoadPlugin(IHeliumHexControl* pHeliumHexControl,IPluginMenu* pPluginMenu);
    virtual void STDMETHODCALLTYPE OnUnloadPlugin();
    virtual BOOL STDMETHODCALLTYPE OnPluginMenuClick(UINT MenuItemID);
    // OnHexDocumentBeforeClose : return FALSE to avoid document closing
    virtual void STDMETHODCALLTYPE OnHexDocumentBeforeClose(IN IHexDocument* pHexDocument);
    // OnHexDocumentAfterClose : pHexDocument mustn't be accessed from now
    virtual void STDMETHODCALLTYPE OnHexDocumentAfterClose(IN IHexDocument* pHexDocument); 
    virtual void STDMETHODCALLTYPE OnHexDocumentAfterOpen(IN IHexDocument* pHexDocument);
    virtual void STDMETHODCALLTYPE OnHexDocumentUnactivate(IN IHexDocument* pHexDocument);
    virtual void STDMETHODCALLTYPE OnHexDocumentActivate(IN IHexDocument* pHexDocument);

    ////////////////////////////////////////////////
    // add your object methods and properties here
    ////////////////////////////////////////////////
    IPluginMenu* pMenu;
    IPluginMenu* pSubMenu1;
    UINT MenuItem0Id;
    UINT SubMenuItem0Id;
    UINT SubMenuItem1Id;
};

CPlugin Plugin;
// the only required dll exported function
extern "C" __declspec(dllexport) IHeliumHexControlEvents* __stdcall GetPluginObject()
{
    return &Plugin;
}

CPlugin::CPlugin()
{
    this->pMenu = NULL;
    this->pSubMenu1 = NULL;
    this->MenuItem0Id = (UINT)-1;
    this->SubMenuItem0Id = (UINT)-1;
    this->SubMenuItem1Id = (UINT)-1;
}
CPlugin::~CPlugin()
{

}

BOOL STDMETHODCALLTYPE CPlugin::OnLoadPlugin(IHeliumHexControl* pHeliumHexControl,IPluginMenu* pPluginMenu)
{
    // call base object init
    this->CHeliumHexControlEventsBase::OnLoadPlugin(pHeliumHexControl,pPluginMenu);

    // do our own plugin init
    this->pMenu = pHeliumHexControl->CreateMenu(pPluginMenu);
    this->pSubMenu1 = pHeliumHexControl->CreateMenu(this->pMenu);

    pPluginMenu->AddSubMenuItem(TEXT("Example"),this->pMenu,LoadIcon(IDI_ICON1));
    this->MenuItem0Id = this->pMenu->AddMenuItem(TEXT("Select All"),LoadIcon(IDI_ICON2));
    this->pMenu->AddSeparatorItem();
    this->pMenu->AddSubMenuItem(TEXT("SubMenu"),this->pSubMenu1,LoadIcon(IDI_ICON3));
    this->SubMenuItem0Id = this->pSubMenu1->AddMenuItem(TEXT("Open Plugin Dll File"),LoadIcon(IDI_ICON4));
    this->SubMenuItem1Id = this->pSubMenu1->AddMenuItem(TEXT("Open Plugin Dll Memory (first 4096 bytes)"),NULL);
    return TRUE;
}


void STDMETHODCALLTYPE CPlugin::OnUnloadPlugin()
{
    // do our own plugin unload
    if (this->pMenu)
    {
        this->pHeliumHexControl->DestroyMenu(this->pMenu);
        this->pMenu = NULL;
    }
    if (this->pSubMenu1)
    {
        this->pHeliumHexControl->DestroyMenu(this->pSubMenu1);
        this->pSubMenu1 = NULL;
    }
}
BOOL STDMETHODCALLTYPE CPlugin::OnPluginMenuClick(UINT MenuItemID)
{
    IHexDocument* pActiveHexDocument = this->pHeliumHexControl->GetActiveHexDocument();
    HWND hWnd= this->pHeliumHexControl->GetHexDocumentViewWindowHandle(pActiveHexDocument);
    if (MenuItemID==this->MenuItem0Id)
    {
        if (pActiveHexDocument)
        {
            // MessageBox(hWnd,TEXT("Plugin Select All"),TEXT("Ex Plugin"),MB_OK|MB_ICONINFORMATION);
            this->pHeliumHexControl->SetSelection(pActiveHexDocument,0,(ULONG64)-1);
        }
        else
            MessageBox(hWnd,TEXT("Can't Select All : no document available"),TEXT("Ex Plugin"),MB_OK|MB_ICONINFORMATION);
        return TRUE;
    }
    else if (MenuItemID==this->SubMenuItem0Id)
    {
        MessageBox(hWnd,TEXT("Open Plugin Dll File"),TEXT("Ex Plugin"),MB_OK|MB_ICONINFORMATION);
        IHeliumHexControl::Open_PARAMETER OpenParam;
        OpenParam.Type = IHeliumHexControl::IHexDocument_TYPE_File;
        OpenParam.File.ReadOnly = TRUE;
        ::GetModuleFileName(DllhInstance,OpenParam.File.FilePath,IHePlugin_OPEN_PARAMETER_MAX_PATH_LEN);
        this->pHeliumHexControl->OpenDocument(&OpenParam);
        return TRUE;
    }
    else if (MenuItemID==this->SubMenuItem1Id)
    {
        MessageBox(hWnd,TEXT("Open Plugin Dll Memory (first 4096 bytes)"),TEXT("Ex Plugin"),MB_OK|MB_ICONINFORMATION);
        IHeliumHexControl::Open_PARAMETER OpenParam;
        OpenParam.Type = IHeliumHexControl::IHexDocument_TYPE_Memory;
        OpenParam.Memory.ReadOnly = TRUE;
        OpenParam.Memory.MemoryStartAddress = (ULONG64)DllhInstance;
        OpenParam.Memory.MemorySize = 4096;
        OpenParam.Memory.ProcessId = ::GetCurrentProcessId();
        this->pHeliumHexControl->OpenDocument(&OpenParam);
        return TRUE;
    }
    return FALSE;
}
// OnHexDocumentBeforeClose : return FALSE to avoid document closing
void STDMETHODCALLTYPE CPlugin::OnHexDocumentBeforeClose(IN IHexDocument* pHexDocument)
{
    UNREFERENCED_PARAMETER(pHexDocument);
} 
// OnHexDocumentAfterClose : pHexDocument mustn't be accessed from now, only reference value can be used
void STDMETHODCALLTYPE CPlugin::OnHexDocumentAfterClose(IN IHexDocument* pHexDocument)
{
    UNREFERENCED_PARAMETER(pHexDocument);
}
void STDMETHODCALLTYPE CPlugin::OnHexDocumentAfterOpen(IN IHexDocument* pHexDocument)
{
    UNREFERENCED_PARAMETER(pHexDocument);
}
void STDMETHODCALLTYPE CPlugin::OnHexDocumentUnactivate(IN IHexDocument* pHexDocument)
{
    UNREFERENCED_PARAMETER(pHexDocument);
}
void STDMETHODCALLTYPE CPlugin::OnHexDocumentActivate(IN IHexDocument* pHexDocument)
{
    UNREFERENCED_PARAMETER(pHexDocument);
}
