#pragma once

#include <windows.h>
#pragma warning (push)
#pragma warning(disable : 4005)// for '_stprintf' : macro redefinition in tchar.h
#include <TCHAR.h>
#pragma warning (pop)

#include "IPluginMenu.h"

typedef HANDLE IHexDocument;
class IHeliumHexControl;

#define IHePlugin_OPEN_PARAMETER_MAX_PATH_LEN 1024

#define IHeliumHexControlPlugin_MAJOR_VERSION 1
#define IHeliumHexControlPlugin_MINOR_VERSION 0

#if ((!defined(UNICODE)) && (!defined(_UNICODE)))
    #error "Only unicode is supported. Please build your project using unicode char encoding"
#endif

// IHePlugin interface
class IHeliumHexControlEvents
{
public:
    SIZE_T virtual STDMETHODCALLTYPE GetPluginFrameworkVersion()
    {
        return ( (IHeliumHexControlPlugin_MAJOR_VERSION<<16) | (IHeliumHexControlPlugin_MINOR_VERSION) );
    }

    virtual BOOL STDMETHODCALLTYPE OnLoadPlugin(IHeliumHexControl* pPluginInterface,IPluginMenu* pPluginMenu)=0;
    virtual void STDMETHODCALLTYPE OnUnloadPlugin()=0;
    virtual BOOL STDMETHODCALLTYPE OnPluginMenuClick(UINT MenuItemID)=0;// should return TRUE if MenuItemID belongs to your plugin menu
    // OnHexDocumentBeforeClose : pHexDocument can be accessed; work can be done before document closing
    virtual void STDMETHODCALLTYPE OnHexDocumentBeforeClose(IN IHexDocument* pHexDocument)=0; 
    // OnHexDocumentAfterClose : pHexDocument mustn't be accessed from now, only reference value can be used (can be used to release memory associated to document)
    virtual void STDMETHODCALLTYPE OnHexDocumentAfterClose(IN IHexDocument* pHexDocument)=0; 
    virtual void STDMETHODCALLTYPE OnHexDocumentAfterOpen(IN IHexDocument* pHexDocument)=0;
    virtual void STDMETHODCALLTYPE OnHexDocumentUnactivate(IN IHexDocument* pHexDocument)=0;
    virtual void STDMETHODCALLTYPE OnHexDocumentActivate(IN IHexDocument* pHexDocument)=0;
};

// CHePluginBase : base class to assume that all events are handled, specific event management should be derivated from this class
class CHeliumHexControlEventsBase:public IHeliumHexControlEvents
{
protected:
    IHeliumHexControl* pHeliumHexControl;
    IPluginMenu* pPluginMenu;
public:
    CHeliumHexControlEventsBase()
    {
        this->pHeliumHexControl = NULL;
        this->pPluginMenu = NULL;
    }
    virtual ~CHeliumHexControlEventsBase()
    {

    }
    virtual BOOL STDMETHODCALLTYPE OnLoadPlugin(IHeliumHexControl* pPluginInterface,IPluginMenu* pPluginMenu)
    {
        this->pHeliumHexControl=pPluginInterface;
        this->pPluginMenu = pPluginMenu;
        return TRUE;
    }
    virtual void STDMETHODCALLTYPE OnUnloadPlugin(){}
    // OnPluginMenuClick : should return TRUE if MenuItemID belongs to your plugin menu
    virtual BOOL STDMETHODCALLTYPE OnPluginMenuClick(UINT MenuItemID){UNREFERENCED_PARAMETER(MenuItemID);return FALSE;}
    // OnHexDocumentBeforeClose : pHexDocument can be accessed; work can be done before document closing
    virtual void STDMETHODCALLTYPE OnHexDocumentBeforeClose(IN IHexDocument* pHexDocument){UNREFERENCED_PARAMETER(pHexDocument);} 
    // OnHexDocumentAfterClose : pHexDocument mustn't be accessed from now, only reference value can be used (can be used to release memory associated to document)
    virtual void STDMETHODCALLTYPE OnHexDocumentAfterClose(IN IHexDocument* pHexDocument){UNREFERENCED_PARAMETER(pHexDocument);}; 
    virtual void STDMETHODCALLTYPE OnHexDocumentAfterOpen(IN IHexDocument* pHexDocument){UNREFERENCED_PARAMETER(pHexDocument);};
    virtual void STDMETHODCALLTYPE OnHexDocumentUnactivate(IN IHexDocument* pHexDocument){UNREFERENCED_PARAMETER(pHexDocument);};
    virtual void STDMETHODCALLTYPE OnHexDocumentActivate(IN IHexDocument* pHexDocument){UNREFERENCED_PARAMETER(pHexDocument);};
};

class IHeliumHexControl
{
public:
    //////////////////////////////////////////////////////
    // structures for open/create types
    //////////////////////////////////////////////////////
    typedef enum IHexDocument_TYPE
    {
        IHexDocument_TYPE_File,
        IHexDocument_TYPE_Memory,
    };

    //////////////////////////////////////////////////////
    // structures for open/create structs
    //////////////////////////////////////////////////////
#pragma pack(push)
#pragma pack(8)
    // for IHexDocument_TYPE_File
    typedef struct tagOpenFile_PARAMETER
    {
        TCHAR FilePath[IHePlugin_OPEN_PARAMETER_MAX_PATH_LEN];    // path of file to be open, *FilePath=0 to let user select file with open file dialog
        BOOL ReadOnly;      // TRUE if read only mode
    }OpenFile_PARAMETER;

    // for IHexDocument_TYPE_Memory
    typedef struct tagOpenMemory_PARAMETER
    {
        DWORD ProcessId;
        ULONG64 MemoryStartAddress;
        ULONG64 MemorySize;
        BOOL ReadOnly;      // TRUE if read only mode
    }OpenMemory_PARAMETER;


    typedef struct tagOpen_PARAMETER
    {
        IHexDocument_TYPE Type;
        union
        {
            // for IHexDocument_TYPE_File
            OpenFile_PARAMETER File;

            // for IHexDocument_TYPE_Memory
            OpenMemory_PARAMETER Memory;
        };
    }Open_PARAMETER;


    // for IHexDocument_TYPE_File
    typedef struct tagNewFile_PARAMETER
    {
        TCHAR FilePath[IHePlugin_OPEN_PARAMETER_MAX_PATH_LEN];    // path of file to be open, *FilePath=0 to let user select file with open file dialog
        SIZE_T Size;
        BOOL bFillWithMotif;
        BYTE Motif;
    }NewFile_PARAMETER;

    // for IHexDocument_TYPE_Memory
    typedef struct tagNewMemory_PARAMETER
    {
        DWORD ProcessId;
        LPVOID PreferredAddress; // 0 if no preferred address
        SIZE_T Size;           
        DWORD flAllocationType; // VirtualAllocEx Param by default should be MEM_COMMIT  (or MEM_COMMIT | MEM_RESERVE if PreferredAddress !=0)
        DWORD flProtect;        // VirtualAllocEx Param
    }NewMemory_PARAMETER;


    typedef struct tagNew_PARAMETER
    {
        IHexDocument_TYPE Type;
        union
        {
            // for IHexDocument_TYPE_File
            NewFile_PARAMETER File;

            // for IHexDocument_TYPE_Memory
            NewMemory_PARAMETER Memory;
        };
    }New_PARAMETER;

#pragma pack(pop)

    virtual IPluginMenu* STDMETHODCALLTYPE CreateMenu(IPluginMenu* pParentMenu)=0;// pParentMenu = NULL for Plug in Menu
    virtual void STDMETHODCALLTYPE DestroyMenu(IPluginMenu* pMenu)=0;

    // get opened hex documents list
    virtual BOOL STDMETHODCALLTYPE GetHexDocuments(OUT IHexDocument** pHexDocumentArray,OUT DWORD* pHexDocumentArraySize)=0;

    // GetActiveHexDocument : get active hex document. Can return NULL if no document opened
    virtual IHexDocument* STDMETHODCALLTYPE GetActiveHexDocument()=0; 
    // SetActiveDocument : set active hex document
    virtual BOOL STDMETHODCALLTYPE SetActiveDocument(IN IHexDocument* pHexDocument)=0;
    // IsHexDocumenStillValid : allow to check if Document is still opened
    virtual BOOL STDMETHODCALLTYPE IsHexDocumenStillValid(IN IHexDocument* pHexDocument)=0;

    virtual IHexDocument* STDMETHODCALLTYPE NewHexDocument(IN New_PARAMETER* pNewHexDocumentInfo)=0;
    virtual IHexDocument* STDMETHODCALLTYPE OpenDocument(IN Open_PARAMETER* pOpenHexDocumentInfo)=0;
    virtual BOOL STDMETHODCALLTYPE CloseHexDocument(IN IHexDocument* pHexDocument)=0;
    virtual BOOL STDMETHODCALLTYPE SaveChanges(IN IHexDocument* pHexDocument)=0;
    // SaveAs : if FilePath is NULL or *FilePath = 0, a dialogs pops up to query file path
    virtual BOOL STDMETHODCALLTYPE SaveAs(IN IHexDocument* pHexDocument,IN OUT WCHAR* FilePath,IN SIZE_T FilePathMaxLen)=0;
    // SaveAsPartial : if FilePath is NULL or *FilePath = 0, a dialogs pops up to query file path
    virtual BOOL STDMETHODCALLTYPE SaveAsPartial(IN IHexDocument* pHexDocument,IN ULONG64 Offset,IN ULONG64 Size,IN OUT WCHAR* FilePath,IN SIZE_T FilePathMaxLen)=0;
    // GetOffsetFromZeroBasedAddress : get hex document offset from 0 based address (for memory,useless for files)
    virtual ULONG64 STDMETHODCALLTYPE GetOffsetFromZeroBasedAddress(IN IHexDocument* pHexDocument,IN ULONG64 ZeroBasedAddress)=0;
    // GetZeroBasedAddressFromOffset : get hex document 0 based address from offset (for memory,useless for files)
    virtual ULONG64 STDMETHODCALLTYPE GetZeroBasedAddressFromOffset(IN IHexDocument* pHexDocument,IN ULONG64 Offset)=0;
    virtual BOOL STDMETHODCALLTYPE GetCaretPosition(IN IHexDocument* pHexDocument,OUT ULONG64* pOffset)=0;
    virtual BOOL STDMETHODCALLTYPE SetCaretPosition(IN IHexDocument* pHexDocument,IN ULONG64 Offset)=0;
    virtual BOOL STDMETHODCALLTYPE GetSelection(IN IHexDocument* pHexDocument,OUT ULONG64* pOffset,OUT ULONG64* pSize)=0;
    virtual BOOL STDMETHODCALLTYPE SetSelection(IN IHexDocument* pHexDocument,IN ULONG64 Offset,IN ULONG64 Size)=0;
    virtual BOOL STDMETHODCALLTYPE GetSize(IN IHexDocument* pHexDocument,OUT ULONG64* pSize)=0;
    virtual BOOL STDMETHODCALLTYPE Read(IN IHexDocument* pHexDocument,IN ULONG64 Offset,IN ULONG64 SizeToRead,OUT PBYTE Buffer)=0;
    // Write : replace data
    virtual BOOL STDMETHODCALLTYPE Write(IN IHexDocument* pHexDocument,IN ULONG64 Offset,IN ULONG64 SizeToWrite,IN PBYTE Buffer)=0;
    virtual BOOL STDMETHODCALLTYPE Insert(IN IHexDocument* pHexDocument,IN ULONG64 Offset,IN PBYTE Buffer,IN ULONG64 BufferSize)=0;
    virtual BOOL STDMETHODCALLTYPE Remove(IN IHexDocument* pHexDocument,IN ULONG64 Offset,IN ULONG64 Size)=0;
    // SetViewStartAddress : set hex document view page start address
    virtual BOOL STDMETHODCALLTYPE SetViewStartAddress(IN IHexDocument* pHexDocument,ULONG64 PageStartAddress)=0;
    // GetViewStartAddress : get hex document view page start address
    virtual ULONG64 STDMETHODCALLTYPE GetViewStartAddress(IN IHexDocument* pHexDocument)=0;
    // GetHexDocumentStartAddress : get hex document start address (for memory, 0 for files)
    virtual ULONG64 STDMETHODCALLTYPE GetHexDocumentStartAddress(IN IHexDocument* pHexDocument)=0;
    // GetHexDocumentStartAddress : get hex document end address
    virtual ULONG64 STDMETHODCALLTYPE GetHexDocumentEndAddress(IN IHexDocument* pHexDocument)=0;
    // GetDescriptionString : get hex document description
    virtual BOOL STDMETHODCALLTYPE GetDescriptionString(IN IHexDocument* pHexDocument,OUT WCHAR* Description,IN DWORD DescriptionMaxLength)=0;
    // GetHexDocumentViewWindowHandle : get hex document view window handle
    virtual HWND STDMETHODCALLTYPE GetHexDocumentViewWindowHandle(IN IHexDocument* pHexDocument)=0;

};
